"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AwsApiParser = void 0;

var _graphqlCompose = require("graphql-compose");

var _AwsService = require("./AwsService");

var _AwsConfigITC = _interopRequireDefault(require("./types/AwsConfigITC"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class AwsApiParser {
  constructor(opts) {
    this.name = opts.name || 'Aws';
    this.awsSDK = opts.awsSDK;
    this._serviceMap = {};
    this.getServicesNames();
  }

  getServicesNames() {
    const serviceNames = Object.keys(this.awsSDK).reduce((res, name) => {
      if (this.awsSDK[name].serviceIdentifier) {
        this._serviceMap[this.awsSDK[name].serviceIdentifier] = name;
        res.push(name);
      }

      return res;
    }, []);
    serviceNames.sort();
    return serviceNames;
  }

  getServiceIdentifier(name) {
    if (this._serviceMap[name]) return name;

    if (!this.awsSDK[name]) {
      throw new Error(`Service with name '${name}' does not exist. Run AwsApiParser.`);
    }

    return this.awsSDK[name].serviceIdentifier;
  }

  getServiceConfig(name) {
    const cfg = this.awsSDK.apiLoader.services[this.getServiceIdentifier(name)];

    if (!cfg) {
      throw new Error(`Service with name '${name}' does not exist.`);
    }

    const versions = Object.keys(cfg);

    if (versions.length === 1) {
      return cfg[versions[0]];
    }

    versions.sort();
    return cfg[versions[versions.length - 1]];
  }

  getService(name) {
    const config = this.getServiceConfig(name); // console.log(config);

    return new _AwsService.AwsService({
      serviceId: this._serviceMap[name] || name,
      prefix: this.name,
      config,
      awsSDK: this.awsSDK
    });
  }

  getTypeComposer() {
    if (!this.tc) {
      const fields = this.getServicesNames().reduce((res, name) => {
        res[this.getServiceIdentifier(name)] = this.getService(name).getFieldConfig();
        return res;
      }, {});
      this.tc = _graphqlCompose.schemaComposer.createObjectTC({
        name: this.name,
        fields,
        description: `AWS SDK ${this.awsSDK.VERSION}`
      });
    }

    return this.tc;
  }

  getType() {
    return this.getTypeComposer().getType();
  }

  getFieldConfig() {
    return {
      type: this.getType(),
      args: {
        config: {
          type: _AwsConfigITC.default.getType(),
          description: 'Will override default configs for aws-sdk.'
        }
      },
      resolve: (source, args) => ({
        awsConfig: _objectSpread({}, source && source.awsConfig, args && args.config)
      }),
      description: this.getTypeComposer().getDescription()
    };
  }

}

exports.AwsApiParser = AwsApiParser;