"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AwsService = void 0;

var _graphqlCompose = require("graphql-compose");

var _AwsServiceMetadata = require("./AwsServiceMetadata");

var _AwsServiceOperation = require("./AwsServiceOperation");

var _AwsShapes = require("./AwsShapes");

var _AwsConfigITC = _interopRequireDefault(require("./types/AwsConfigITC"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class AwsService {
  constructor(opts) {
    this.prefix = opts.prefix;
    this.serviceId = opts.serviceId;
    this.awsSDK = opts.awsSDK;
    this.config = opts.config;
    this.metadata = new _AwsServiceMetadata.AwsServiceMetadata(this.config.metadata);
    this.shapes = new _AwsShapes.AwsShapes(this.config.shapes, this.getTypeName());
  }

  getTypeName() {
    return `${this.prefix}${(0, _graphqlCompose.upperFirst)(this.serviceId)}`;
  }

  static lowerFirst(name) {
    return name.charAt(0).toLowerCase() + name.slice(1);
  }

  getOperationNames() {
    return Object.keys(this.config.operations);
  }

  getOperation(name) {
    const operConfig = this.config.operations[name];

    if (!operConfig) {
      throw new Error(`Operation with name ${name} does not exist.`);
    }

    return new _AwsServiceOperation.AwsServiceOperation({
      serviceId: this.serviceId,
      name: this.constructor.lowerFirst(name),
      prefix: this.prefix,
      config: operConfig,
      shapes: this.shapes,
      awsSDK: this.awsSDK
    });
  }

  getTypeComposer() {
    if (!this.tc) {
      const fields = this.getOperationNames().reduce((res, name) => {
        res[this.constructor.lowerFirst(name)] = this.getOperation(name).getFieldConfig();
        return res;
      }, {});
      this.tc = _graphqlCompose.schemaComposer.createObjectTC({
        name: this.getTypeName(),
        fields,
        description: this.metadata.getDescription()
      });
    }

    return this.tc;
  }

  getType() {
    return this.getTypeComposer().getType();
  }

  getFieldConfig() {
    return {
      type: this.getType(),
      args: {
        config: {
          type: _AwsConfigITC.default.getType(),
          description: 'Will override default configs for aws-sdk.'
        }
      },
      resolve: (source, args) => ({
        awsConfig: _objectSpread({}, source && source.awsConfig, args && args.config)
      })
    };
  }

}

exports.AwsService = AwsService;