"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AwsServiceOperation = void 0;

var _graphqlCompose = require("graphql-compose");

var _AwsParam = require("./AwsParam");

var _AwsConfigITC = _interopRequireDefault(require("./types/AwsConfigITC"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class AwsServiceOperation {
  constructor(opts) {
    this.prefix = opts.prefix;
    this.name = opts.name;
    this.serviceId = opts.serviceId;
    this.awsSDK = opts.awsSDK;
    this.config = opts.config;
    this.shapes = opts.shapes;
  }

  getTypeName() {
    return `${this.prefix}${(0, _graphqlCompose.upperFirst)(this.serviceId)}${(0, _graphqlCompose.upperFirst)(this.name)}`;
  }

  getType() {
    if (this.config.output && this.config.output.type === 'structure') {
      const tc = _AwsParam.AwsParam.convertOutputStructure(this.config.output, `${this.getTypeName()}Output`, this.shapes); // If bugous structure without fields, then return JSON


      if (Object.keys(tc.getFields()).length === 0) {
        return _graphqlCompose.GraphQLJSON;
      }

      return tc.getType();
    }

    return _graphqlCompose.GraphQLJSON;
  }

  getArgs() {
    if (!this.args) {
      if (!this.config.input) {
        this.args = {};
      } else {
        const itc = _AwsParam.AwsParam.convertInputStructure(this.config.input, this.getTypeName(), this.shapes);

        let type; // If bugous structure without fields, then return JSON

        if (Object.keys(itc.getFields()).length === 0) {
          type = _graphqlCompose.GraphQLJSON;
        } else {
          const hasRequiredFields = itc.getFieldNames().some(f => itc.isRequired(f));
          type = hasRequiredFields ? itc.getTypeNonNull() : itc.getType();
        }

        this.args = {
          input: {
            type
          }
        };
      }

      this.args.config = {
        type: _AwsConfigITC.default.getType()
      };
    }

    return this.args;
  }

  getResolve() {
    return (source, args) => {
      return new Promise((resolve, reject) => {
        const awsConfig = _objectSpread({}, source && source.awsConfig, args && args.config);

        const service = new this.awsSDK[this.serviceId](awsConfig);
        service[this.name](args.input, (err, data) => {
          if (err) {
            reject(err);
          }

          resolve(data);
        });
      });
    };
  }

  getFieldConfig() {
    return {
      type: this.getType(),
      args: this.getArgs(),
      resolve: this.getResolve()
    };
  }

}

exports.AwsServiceOperation = AwsServiceOperation;