"use strict";

var _awsSdk = _interopRequireDefault(require("aws-sdk"));

var _AwsApiParser = require("../AwsApiParser");

var _AwsService = require("../AwsService");

var _AwsConfigITC = _interopRequireDefault(require("../types/AwsConfigITC"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('AwsApiParser', () => {
  const aws = new _AwsApiParser.AwsApiParser({
    awsSDK: _awsSdk.default
  });
  it('getServicesNames()', () => {
    const names = aws.getServicesNames();
    expect(names).toContain('S3');
    expect(names).toContain('EC2');
    expect(names).toContain('Route53');
    expect(names).toContain('SQS');
  });
  it('getServiceIdentifier()', () => {
    expect(aws.getServiceIdentifier('S3')).toBe('s3'); // also should work if provided serviceIdentifier

    expect(aws.getServiceIdentifier('s3')).toBe('s3');
  });
  it('getServiceConfig()', () => {
    const cfg = aws.getServiceConfig('S3');
    expect(Object.keys(cfg)).toEqual(['version', 'metadata', 'operations', 'shapes', 'paginators', 'waiters']); // get config by serviceIdentifier

    const cfg2 = aws.getServiceConfig('s3');
    expect(Object.keys(cfg2)).toEqual(['version', 'metadata', 'operations', 'shapes', 'paginators', 'waiters']);
  });
  it('getService()', () => {
    const service = aws.getService('S3');
    expect(service).toBeInstanceOf(_AwsService.AwsService); // get service by serviceIdentifier

    const service2 = aws.getService('s3');
    expect(service2).toBeInstanceOf(_AwsService.AwsService);
  });
  it('getType()', () => {
    const type = aws.getType();
    expect(type.name).toBe('Aws');
  });
  it('getFieldConfig()', () => {
    const fc = aws.getFieldConfig();
    expect(fc.type).toBe(aws.getType());
    expect(fc.args.config.type).toBe(_AwsConfigITC.default.getType());
    expect(fc.resolve()).toEqual({
      awsConfig: {}
    });
  });
});