"use strict";

var _graphql = require("graphql-compose/lib/graphql");

var _AwsServiceOperation = require("../AwsServiceOperation");

var _AwsConfigITC = _interopRequireDefault(require("../types/AwsConfigITC"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const operations = {
  CreateBucket: {
    http: {
      method: 'PUT',
      requestUri: '/{Bucket}'
    },
    input: {
      type: 'structure',
      required: ['Bucket'],
      members: {
        ACL: {
          location: 'header',
          locationName: 'x-amz-acl'
        },
        Bucket: {
          location: 'uri',
          locationName: 'Bucket'
        },
        CreateBucketConfiguration: {
          locationName: 'CreateBucketConfiguration',
          xmlNamespace: {
            uri: 'http://s3.amazonaws.com/doc/2006-03-01/'
          },
          type: 'structure',
          members: {
            LocationConstraint: {}
          }
        },
        GrantFullControl: {
          location: 'header',
          locationName: 'x-amz-grant-full-control'
        },
        GrantRead: {
          location: 'header',
          locationName: 'x-amz-grant-read'
        },
        GrantReadACP: {
          location: 'header',
          locationName: 'x-amz-grant-read-acp'
        },
        GrantWrite: {
          location: 'header',
          locationName: 'x-amz-grant-write'
        },
        GrantWriteACP: {
          location: 'header',
          locationName: 'x-amz-grant-write-acp'
        }
      },
      payload: 'CreateBucketConfiguration'
    },
    output: {
      type: 'structure',
      members: {
        Location: {
          location: 'header',
          locationName: 'Location'
        }
      }
    },
    alias: 'PutBucket'
  }
};
describe('AwsJsonParserOperation', () => {
  const AWSMock = {
    S3: class S3 {
      CreateBucket(args, cb) {} // eslint-disable-line


    }
  };
  const oper = new _AwsServiceOperation.AwsServiceOperation({
    prefix: 'AWS',
    serviceId: 'S3',
    name: 'CreateBucket',
    config: operations.CreateBucket,
    awsSDK: AWSMock,
    shapes: {}
  });
  it('getTypeName()', () => {
    expect(oper.getTypeName()).toBe('AWSS3CreateBucket');
  });
  it('getType()', () => {
    const tc = oper.getType();
    expect((0, _graphql.isOutputType)(tc)).toBeTruthy();
  });
  it('getArgs()', () => {
    const args = oper.getArgs();
    expect(args.input).toBeDefined(); // input arg has required fields, so it wrapped by GraphQLNonNull

    expect(args.input.type).toBeInstanceOf(_graphql.GraphQLNonNull);
    expect(args.input.type.ofType).toBeInstanceOf(_graphql.GraphQLInputObjectType);
    expect(args.input.type.ofType.name).toBe('AWSS3CreateBucketInput');
  });
  describe('getResolve()', () => {
    const resolve = oper.getResolve();
    it('to be function', () => {
      expect(resolve.call).toBeDefined();
    });
    it('resolves',
    /*#__PURE__*/
    _asyncToGenerator(function* () {
      AWSMock.S3 = class S3WithPayload {
        CreateBucket(args, cb) {
          cb(undefined, {
            payload: args
          });
        }

      };
      const res = yield resolve({}, {
        input: {
          arg: 123
        }
      }, {}, {});
      expect(res).toEqual({
        payload: {
          arg: 123
        }
      });
    }));
    it('rejects',
    /*#__PURE__*/
    _asyncToGenerator(function* () {
      AWSMock.S3 = class S3WithError {
        CreateBucket(args, cb) {
          cb('err');
        }

      };
      expect.assertions(1);

      try {
        yield resolve({}, {}, {}, {});
      } catch (e) {
        expect(e).toBe('err');
      }
    }));
  });
  it('getFieldConfig()', () => {
    const fc = oper.getFieldConfig();
    expect(fc.type).toBeInstanceOf(_graphql.GraphQLObjectType);
    expect(fc.args.config.type).toBe(_AwsConfigITC.default.getType());
    expect(fc.resolve.call).toBeDefined();
  });
});