import { schemaComposer, ObjectTypeComposer, InputTypeComposer, upperFirst } from 'graphql-compose';
export class AwsParam {
  static convertInputStructure(param, name, shapes) {
    const fields = {};

    if (param.members) {
      Object.keys(param.members).forEach(fname => {
        fields[fname] = this.convertParam(param.members[fname], `${name}${upperFirst(fname)}`, true, shapes);
      });
    }

    const itc = schemaComposer.createInputTC({
      name: `${name}Input`,
      fields
    });

    if (Array.isArray(param.required)) {
      itc.makeRequired(param.required);
    }

    return itc;
  }

  static convertOutputStructure(param, name, shapes) {
    const fields = {};

    if (param.members) {
      Object.keys(param.members).forEach(fname => {
        fields[fname] = this.convertParam(param.members[fname], `${name}${upperFirst(fname)}`, false, shapes);
      });
    }

    const tc = schemaComposer.createObjectTC({
      name,
      fields
    });

    if (Array.isArray(param.required)) {
      tc.makeFieldNonNull(param.required);
    }

    return tc;
  }

  static convertParam(param, name, isInput, shapes) {
    if (param.type) {
      switch (param.type) {
        case 'boolean':
          return 'Boolean';

        case 'string':
          return 'String';

        case 'integer':
          return 'Int';

        case 'float':
          return 'Float';

        case 'timestamp':
          return 'Date';

        case 'structure':
          {
            const tc = isInput ? this.convertInputStructure(param, name, shapes) : this.convertOutputStructure(param, name, shapes); // If bugous structure without fields, then return JSON

            if (Object.keys(tc.getFields()).length === 0) {
              return 'JSON';
            }

            return tc;
          }

        case 'map':
          return 'JSON';

        case 'list':
          // $FlowFixMe
          return [this.convertParam(param.member, name, isInput, shapes)];

        default:
          return 'JSON';
      }
    }

    if (param.shape) {
      return this.convertShape(param, isInput, shapes);
    }

    return 'String';
  }

  static convertShape(param, isInput, shapes) {
    if (shapes) {
      return isInput ? shapes.getInputShape(param.shape) : shapes.getOutputShape(param.shape);
    }

    return 'JSON';
  }

}