function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { schemaComposer, ObjectTypeComposer, upperFirst } from 'graphql-compose';
import { AwsServiceMetadata } from './AwsServiceMetadata';
import { AwsServiceOperation } from './AwsServiceOperation';
import { AwsShapes } from './AwsShapes';
import AwsConfigITC from './types/AwsConfigITC';
export class AwsService {
  constructor(opts) {
    this.prefix = opts.prefix;
    this.serviceId = opts.serviceId;
    this.awsSDK = opts.awsSDK;
    this.config = opts.config;
    this.metadata = new AwsServiceMetadata(this.config.metadata);
    this.shapes = new AwsShapes(this.config.shapes, this.getTypeName());
  }

  getTypeName() {
    return `${this.prefix}${upperFirst(this.serviceId)}`;
  }

  static lowerFirst(name) {
    return name.charAt(0).toLowerCase() + name.slice(1);
  }

  getOperationNames() {
    return Object.keys(this.config.operations);
  }

  getOperation(name) {
    const operConfig = this.config.operations[name];

    if (!operConfig) {
      throw new Error(`Operation with name ${name} does not exist.`);
    }

    return new AwsServiceOperation({
      serviceId: this.serviceId,
      name: this.constructor.lowerFirst(name),
      prefix: this.prefix,
      config: operConfig,
      shapes: this.shapes,
      awsSDK: this.awsSDK
    });
  }

  getTypeComposer() {
    if (!this.tc) {
      const fields = this.getOperationNames().reduce((res, name) => {
        res[this.constructor.lowerFirst(name)] = this.getOperation(name).getFieldConfig();
        return res;
      }, {});
      this.tc = schemaComposer.createObjectTC({
        name: this.getTypeName(),
        fields,
        description: this.metadata.getDescription()
      });
    }

    return this.tc;
  }

  getType() {
    return this.getTypeComposer().getType();
  }

  getFieldConfig() {
    return {
      type: this.getType(),
      args: {
        config: {
          type: AwsConfigITC.getType(),
          description: 'Will override default configs for aws-sdk.'
        }
      },
      resolve: (source, args) => ({
        awsConfig: _objectSpread({}, source && source.awsConfig, args && args.config)
      })
    };
  }

}